% The program for the article "Public Liquidity, Bank Runs, and Financial Crises", by Wenhao Li.
% This file generates model solutions, basic model output and provides replications on figures and tables in the paper. 
% Created by Wenhao Li, www.wenhao-li.com, on Feb 22, 2018.

%% ----------------------------------------------------------------
%                                           Basic Settings
% ------------------------------------------------------------------------
clear all; close all; clc;
mainpath = '/Users/wenhao/Dropbox/Research/LiqInsurance_GovBond_CleanFolder/Model_baseline';  
cd(mainpath);
addpath(genpath( 'Codes_Common Components'))   % Add other commonly shared files to the path. 
% plot related settings
set(0,'defaultAxesFontSize',16);

%% ----------------------------------------------------------------
%                                           Initialization
% ------------------------------------------------------------------------
% ************ (1) Model Parameters ************
% Productivity
AH=0.15;  AL=0.125;  
% Bank runs and fire sales
lambda=0.025;    beta = 0.25;    alpha0  = 0.21;     
% Asset pricing
PI = 0.19;
% Standard macro parameters
delta=0.1;    rho = 0.04;     sigmaK=0.033;   % Note: might restore to 0.033 if things wont' work out.
chi=3;
investment =  @(p)  (p-1).^2/(2*chi) + (p-1)./chi + delta ;    %  This is the investment adjustment cost, or the real resources consumed.
investmentD = @(p)  p/chi ;     
muK_fun = @(p) (p-1)/chi  + delta;   % capital growth (already solved)
% Others
eta = 0.05;   theta=10^(-5);    e = 10^(-3);  

% ************ (2) Computation of Related Parameters ************
% A damping function to avoid corners. This can be extended to allow for household special demand of liquidity.
v0 =3e-8;   v_prime_fun = @(yg)  v0./yg;  
% Average GDP, used as a scalar.
Avg_GDP = 0.145;  
% Average Liquidity/GDP ratio in the data. 
liq_GDP_ratio = 0.39;
% Grid sidze
N_w    = 100; 
% Maximum iterations
parameters.max_it = 12;
% Simulation interval in term of years. This is a monthly simulation.
dt = 1/12;

% ************ (3) Store Parameters ************
parameters.AH=AH;    parameters.AL=  AL; 
parameters.lambda=lambda;   parameters.beta = beta;      parameters.alpha0  = alpha0 ;      
parameters.PI = PI;     
parameters.delta=delta;  parameters.rho = rho;   parameters.sigmaK=sigmaK;    parameters.chi=chi;
parameters.investment =  investment; parameters.investmentD = investmentD;    parameters.muK_fun=muK_fun;
parameters.eta = eta;  parameters.theta=theta;   parameters.e = e;   
parameters.v_prime_fun = v_prime_fun;  
parameters.Avg_GDP=Avg_GDP;
parameters.N_w    = N_w;       parameters.dt = dt;       
parameters_default = parameters;   

% ************ (4) Grid for Different Liquidity Values and Liquidity Functions ************ 
GovBond_GDP_Level = 0.3;  
Q_multipliers =GovBond_GDP_Level * Avg_GDP;  
N_Q = numel(Q_multipliers); % liquidity supply as a fraction of GDP
% print_level: 0 (no printing), 1 (printing), 2(diagnose the inside solution function)
print_level=1;    

%% ----------------------------------------------------------------
%                            Baseline Solution
% ------------------------------------------------------------------------
parameters.max_it = 30;   % maxum number of iterations
getenv('TZ');  % Change the time zone settings to supress warnings. No practical meaning.
initialization_list = cell(N_Q,1); 
parameters_list = cell(N_Q,1); 
filename = [   'solutions/baseline.mat' ];
parameters.Q = @(w)   GovBond_GDP_Level*Avg_GDP*ones( size(w) ); 
disp( [ '----------------------------------  ---------------------------------------------------'  ]  );
disp( [ '                             Solving the Model with Liq/GDP Ratio=', num2str(GovBond_GDP_Level) ]  );
disp( [ '-------------------------------------------------------------------------------------'  ]  );
model_solution = find_solution( parameters,  filename,  print_level  );



